/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/sch-rnd
 *    contact lead developer: http://www.repo.hu/projects/sch-rnd/contact.html
 *    mailing list: http://www.repo.hu/projects/sch-rnd/contact.html
 */

#ifndef OSM_UNDO_H
#define OSM_UNDO_H

#include "data.h"

typedef enum csch_undo_ev_e {
	OSM_UNDO_EV_UNDO,
	OSM_UNDO_EV_REDO,
	OSM_UNDO_EV_CLEAR_LIST,
	OSM_UNDO_EV_TRUNCATE
} csch_undo_ev_t;

int osm_undo(osm_layer_t *layer);
int osm_redo(osm_layer_t *layer);
void osm_undo_clear_list(osm_layer_t *layer, rnd_bool Force);
void osm_undo_inc_serial(osm_layer_t *layer);

void osm_undo_act_init(void);
void osm_undo_act_uninit(void);

/* Helper for debugging undo bugs: place a visible mark in the undo list */
#define osm_undo_add_mark(layer, comment) \
	do { \
		osm_source_arg_t *src = osm_attrib_src_c(NULL, 0, 0, NULL); \
		osm_attr_modify_str(layer, &layer->direct, 0, comment, NULL, src, 1); \
	} while(0)


/*** helpers for implementing undo swaps ***/

/* Returns arg if not NULL, else obj->field; field and arg have the same type
   but arg is a pointer. */
#define OSM_UNDO_MODIFY_ARG(obj, field, arg)  (arg == NULL ? (obj)->curr->field : *arg)
#define OSM_UNDO_MODIFY(obj, field)           OSM_UNDO_MODIFY_ARG(obj, field, field)

#endif
