/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2022, 2023 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "config.h"
#include <librnd/config.h>

#include <string.h>

#include <librnd/core/event.h>
#include <librnd/core/actions.h>
#include <librnd/core/hidlib.h>
#include <librnd/core/rnd_conf.h>
#include <librnd/core/compat_misc.h>
#include <librnd/core/compat_fs.h>
#include <librnd/core/globalconst.h>
#include <librnd/core/plugins.h>
#include <librnd/plugins/lib_hid_common/dialogs_conf.h>
#include <librnd/plugins/lib_hid_common/dlg_pref.h>
#include <librnd/plugins/lib_hid_common/dlg_export.h>

#include "event.h"

#include "dlg_about.h"
#include "dlg_tags.h"
#include "dlg_undo.h"

static const char *osm_dialogs_cookie = "osm_dialogs";

static rnd_action_t osm_dialogs_action_list[] = {
	{"About", osm_act_About, osm_acth_About, osm_acts_About},
	{"UndoDialog", osm_act_UndoDialog, osm_acth_UndoDialog, osm_acts_UndoDialog},
	{"TagsDialog", osm_act_TagsDialog, osm_acth_TagsDialog, osm_acts_TagsDialog}
};

extern int osm_dlg_pref_tab;
extern void (*osm_dlg_pref_first_init)(pref_ctx_t *ctx, int tab);

int pplg_check_ver_dialogs(int ver_needed) { return 0; }

void pplg_uninit_dialogs(void)
{
	rnd_event_unbind_allcookie(osm_dialogs_cookie);
	rnd_remove_actions_by_cookie(osm_dialogs_cookie);
	rnd_dlg_pref_uninit();
	osm_dlg_undo_uninit();
}

int pplg_init_dialogs(void)
{
	RND_API_CHK_VER;

	RND_REGISTER_ACTIONS(osm_dialogs_action_list, osm_dialogs_cookie);
	rnd_dlg_pref_init(osm_dlg_pref_tab, osm_dlg_pref_first_init);
	osm_dlg_undo_init();

	return 0;
}
