/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - Optimizations
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

void osm_opt_lineup_search(osm_layer_t *ly, vtp0_t *node_list)
{
	vtp0_t tmp = {0};
	long n, m;

	osm_search_all_selected_mask(&tmp, ly, OSM_MASK_NODE | OSM_MASK_WAY);

	for(n = 0; n < tmp.used; n++) {
		osm_any_obj_t *obj = tmp.array[n];
		switch(obj->hdr.type) {
			case OSM_OBJ_NODE:
				if (!obj->hdr.MARK) {
					vtp0_append(node_list, obj);
					obj->hdr.MARK = 1;
				}
				break;
			case OSM_OBJ_WAY: /* add all nodes */
				for(m = 0; m < obj->way.ndp.used; m++) {
					osm_node_t *nd = obj->way.ndp.array[m];
					if (!nd->hdr.MARK) {
						vtp0_append(node_list, nd);
						nd->hdr.MARK = 1;
					}
				}
				break;
		}
	}

	vtp0_uninit(&tmp);
}


static int osm_opt_lineup_x(osm_layer_t *ly, vtp0_t *node_list)
{
	long n;

	if (node_list->used > 2) {
		rnd_coord_t min_x, max_x, min_y, max_y, lx1, ly1, lx2, ly2;
		osm_node_t *min_x_nd, *min_y_nd, *max_x_nd, *max_y_nd;

		min_x = min_y = RND_COORD_MAX;
		max_x = max_y = -RND_COORD_MAX;

		/* find minimums and maximums */
		for(n = 0; n < node_list->used; n++) {
			osm_node_t *nd = node_list->array[n];
			if (nd->x < min_x) {
				min_x = nd->x;
				min_x_nd = nd;
			}
			if (nd->x > max_x) {
				max_x = nd->x;
				max_x_nd = nd;
			}
			if (nd->y < min_y) {
				min_y = nd->y;
				min_y_nd = nd;
			}
			if (nd->y > max_y) {
				max_y = nd->y;
				max_y_nd = nd;
			}
		}

		/* Figure two points on the line */
		if ((max_x - min_x) > (max_y - min_y)) {
			lx1 = min_x_nd->x;
			ly1 = min_x_nd->y;
			lx2 = max_x_nd->x;
			ly2 = max_x_nd->y;
			rnd_trace(" x case\n");
		}
		else {
			lx1 = min_y_nd->x;
			ly1 = min_y_nd->y;
			lx2 = max_y_nd->x;
			ly2 = max_y_nd->y;
			rnd_trace(" y case\n");
		}

		rnd_trace("line: %ld;%ld %ld;%ld\n", lx1, ly1, lx2, ly2);

		/* project each point onto the line */
		for(n = 0; n < node_list->used; n++) {
			osm_node_t *nd = node_list->array[n];
			rnd_coord_t nx, ny;

			osm_proj_pt_onto_line(&nx, &ny, nd->x, nd->y, lx1, ly1, lx2, ly2);
			if ((nx != nd->x) || (ny != nd->y)) {
				double lon, lat;
				rnd2lldeg(&lon, &lat, nx, ny);
				if ((lon != nd->curr->lon) || (lat != nd->curr->lat))
					osm_node_modify(ly, nd, &lon, &lat, 1);
			}
		}
	}

	/* remove marks */
	for(n = 0; n < node_list->used; n++) {
		osm_node_t *nd = node_list->array[n];
		nd->hdr.MARK = 0;
	}

	return 0;
}

void osm_opt_lineup(osm_layer_t *ly, vtp0_t *node_list)
{
	uundo_freeze_serial(&ly->undo);

	osm_opt_lineup_x(ly, node_list);

	uundo_unfreeze_serial(&ly->undo);
	uundo_inc_serial(&ly->undo);
}


